/////////////////////////////////////////////////////////////////////////////
// Name:        TM.cpp
// Purpose:     Classes used to define Turing machines
// Author:      Matthew Patitz <mpatitz@yahoo.com>
// Modified by:
// Created:     Nov/2007
// Copyright:   (c) 2008, Matthew John Patitz
// Licence:     GPL
/////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "TM.h"

extern char* ReservedChars[];

State::State(const char* sIn)
        : std::string(sIn)
{
}

bool State::IsValid()
{
  int i = 0;
  while (ReservedChars[i]) {
    if (find(ReservedChars[i]) != -1) {
      printf("\nError: '%s' is a reserved character and cannot be used in the name of a state.\n", ReservedChars[i]);
      return false;
    }
    i++;
  }
  return true;
}

bool StateSet::IsValid()
{
  for (size_t i=0; i<size(); i++) {
    if (!at(i).IsValid()) {
      return false;
    }
  }
  return true;
}

bool Alphabet::IsValid()
{
  for (size_t j=0; j<size(); j++) {
    int i = 0;
    while (ReservedChars[i]) {
      if ((at(j).find(ReservedChars[i]) != -1) && (at(j) != BLANK)) {
        printf("\nError: '%s' is a reserved character and cannot be used in the input alphabet.\n", at(j).c_str());
        return false;
      }
      i++;
    }
  }
  return true;
}

Transition::Transition()
{
  m_direction = INVALID;
}

bool Transition::IsValid()
{
  if (m_direction != LEFT && m_direction != RIGHT) {
    printf("Error:  Invalid direction specified for transition!\n");
    return false;
  }
  if (m_startState.length() == 0 || m_input.length() == 0 ||
      m_endState.length()   == 0 || m_output.length() == 0)
  {
    printf("Error:  Invalid transition - at least one item is missing from the definition!\n");
    return false;
  }
  return true;
}

bool TuringMachine::IsValid()
{
  if (m_startState.length() == 0) {
    printf("Error:  No start state specified for Turing machine!\n");
    return false;
  } else if (m_alphabet.size() == 0) {
    printf("Error:  No alphabet symbols specified for Turing machine!\n");
    return false;
  } else if (m_states.size() == 0) {
    printf("Error:  No states specified for Turing machine!\n");
    return false;
  } else if (m_transitions.size() == 0) {
    printf("Error:  No transitions specified for Turning machine!\n");
    return false;
  }
  if (!m_alphabet.IsValid()) {
    return false;
  }
  if (!m_states.IsValid()) {
    return false;
  }
  for (size_t i=0; i<m_transitions.size(); i++) {
    if (!m_transitions[i].IsValid()) {
      return false;
    }
  }
  return true;
}
