/////////////////////////////////////////////////////////////////////////////
// Name:        TM.h
// Purpose:     Classes used to define Turing machines
// Author:      Matthew Patitz <mpatitz@yahoo.com>
// Modified by:
// Created:     Nov/2007
// Copyright:   (c) 2008, Matthew John Patitz
// Licence:     GPL
/////////////////////////////////////////////////////////////////////////////

#ifndef __TM_H__
#define __TM_H__

#include <string>
#include <vector>


#define INVALID  -1
#define LEFT      0
#define RIGHT     1

#define BLANK       "_"


class State : public std::string
{
public:
  State(const char* sIn);
  State() { State(""); };

  bool IsValid();
};

class StateSet : public std::vector<State>
{
public:
  bool IsValid();
};

class Alphabet : public std::vector<std::string>
{
public:
  bool IsValid();
};

class Transition
{
public:
  Transition();
  ~Transition() {};
  bool IsValid();

  State         m_startState;
  std::string   m_input;

  State         m_endState;
  std::string   m_output;
  int           m_direction;
};
class TransitionSet : public std::vector<Transition>
{
};

class TuringMachine
{
public:
  bool IsValid();

  StateSet        m_states;
  Alphabet        m_alphabet;
  std::string     m_startState;
  std::string     m_haltingState;
  std::string     m_acceptingState;
  std::string     m_rejectingState;
  TransitionSet   m_transitions;
};

#endif  // __TM_H__