/////////////////////////////////////////////////////////////////////////////
// Name:        counters.cpp
// Purpose:     Generate TAM tilesets which self-assemble into counters
// Author:      Matthew Patitz <mpatitz@yahoo.com>
// Modified by:
// Created:     Nov/2007
// Copyright:   (c) 2008, Matthew John Patitz
// Licence:     GPL
/////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "counters.h"


bool GenerateCounterTileSet(int iBase, int iDirGrow, int iDirFace, int iType, TileSet* pTS)
{

  if ((iDirFace == iDirGrow) || (iDirFace == ((iDirGrow + 2) % 4))) {
    std::cout << "\n\n\nERROR:  Invalid direction specified for 'face' argument.\n\n";
    return false;
  }

  TileSet ts;

  if (iType == COUNTER_LOGWIDTH) {
    //  First tile which is 0 and both MOSTSIG and LEASTSIG
    Tile b("SEED");
    b.m_display                 = MOSTSIG + itos(0) + MARKER + LEASTSIG;
    b.m_sides[NORTH].m_strength = 2;
    b.m_sides[EAST].m_strength  = 0;
    b.m_sides[SOUTH].m_strength = 0;
    b.m_sides[WEST].m_strength  = 0;
    b.m_sides[NORTH].m_label    = MOSTSIG + itos(0) + MARKER + LEASTSIG;
    b.m_sides[EAST].m_label     = NODISPLAY;
    b.m_sides[SOUTH].m_label    = NODISPLAY;
    b.m_sides[WEST].m_label     = NODISPLAY;
    ts.push_back(b);

    for (int j=1; j<iBase; j++) {
      //  First iBase-2 tiles which are both MOSTSIG and LEASTSIG
      Tile b;
      b.m_display                 = MOSTSIG + itos(j) + MARKER + LEASTSIG;
      b.m_sides[NORTH].m_strength = 2;
      b.m_sides[EAST].m_strength  = 0;
      b.m_sides[SOUTH].m_strength = 2;
      b.m_sides[WEST].m_strength  = 0;
      b.m_sides[NORTH].m_label    = MOSTSIG + itos(j) + MARKER + LEASTSIG;
      b.m_sides[EAST].m_label     = NODISPLAY;
      b.m_sides[SOUTH].m_label    = MOSTSIG + itos(j-1) + MARKER + LEASTSIG;
      b.m_sides[WEST].m_label     = NODISPLAY;
      ts.push_back(b);
    }
  } else {
    //  Seed tiles for fixed-width counters
    Tile b("SEEDL");
    b.m_display                 = std::string("S0") + LEASTSIG;
    b.m_sides[NORTH].m_strength = 2;
    b.m_sides[EAST].m_strength  = 0;
    b.m_sides[SOUTH].m_strength = 0;
    b.m_sides[WEST].m_strength  = 1;
    b.m_sides[NORTH].m_label    = itos(0) + MARKER + LEASTSIG;
    b.m_sides[EAST].m_label     = NODISPLAY;
    b.m_sides[SOUTH].m_label    = NODISPLAY;
    b.m_sides[WEST].m_label     = SEED;
    ts.push_back(b);

    Tile c("SEEDINT");
    c.m_display                 = "S" + itos(0);
    c.m_sides[NORTH].m_strength = 1;
    c.m_sides[EAST].m_strength  = 1;
    c.m_sides[SOUTH].m_strength = 0;
    c.m_sides[WEST].m_strength  = 1;
    c.m_sides[NORTH].m_label    = itos(0);
    c.m_sides[EAST].m_label     = SEED;
    c.m_sides[SOUTH].m_label    = NODISPLAY;
    c.m_sides[WEST].m_label     = SEED;
    ts.push_back(c);

    Tile d("SEEDM");
    d.m_display                 = std::string(MOSTSIG) + "S" + itos(0);
    d.m_sides[NORTH].m_strength = 1;
    d.m_sides[EAST].m_strength  = 1;
    d.m_sides[SOUTH].m_strength = 0;
    d.m_sides[WEST].m_strength  = 0;
    d.m_sides[NORTH].m_label    = MOSTSIG + itos(0);
    d.m_sides[EAST].m_label     = SEED;
    d.m_sides[SOUTH].m_label    = NODISPLAY;
    d.m_sides[WEST].m_label     = NODISPLAY;
    ts.push_back(d);
  }

  if (iType == COUNTER_LOGWIDTH) {
    //  The first tile to split MSB and LSB
    Tile e;
    e.m_display                 = itos(0) + MARKER + LEASTSIG;
    e.m_sides[NORTH].m_strength = 2;
    e.m_sides[EAST].m_strength  = 0;
    e.m_sides[SOUTH].m_strength = 2;
    e.m_sides[WEST].m_strength  = 2;
    e.m_sides[NORTH].m_label    = itos(0) + MARKER + LEASTSIG;
    e.m_sides[EAST].m_label     = NODISPLAY;
    e.m_sides[SOUTH].m_label    = MOSTSIG + itos(iBase-1) + MARKER + LEASTSIG;
    e.m_sides[WEST].m_label     = MOSTSIG;
    ts.push_back(e);
  }

  //  The first tile in a new MSB column
  Tile f;
  f.m_display                 = MOSTSIG + itos(1);
  f.m_sides[NORTH].m_strength = 1;
  f.m_sides[EAST].m_strength  = 2;
  f.m_sides[SOUTH].m_strength = 0;
  f.m_sides[WEST].m_strength  = 0;
  f.m_sides[NORTH].m_label    = MOSTSIG + itos(1);
  f.m_sides[EAST].m_label     = MOSTSIG;
  f.m_sides[SOUTH].m_label    = NODISPLAY;
  f.m_sides[WEST].m_label     = MOSTSIG;
  ts.push_back(f);

  //  Tile in LSB position with marker and value (iBase-1) and passing MARKER left
  Tile g;
  g.m_display                 = itos(iBase-1) + LEASTSIG;
  g.m_sides[NORTH].m_strength = 1;
  g.m_sides[EAST].m_strength  = 0;
  g.m_sides[SOUTH].m_strength = 2;
  g.m_sides[WEST].m_strength  = 1;
  g.m_sides[NORTH].m_label    = itos(iBase-1) + LEASTSIG;
  g.m_sides[EAST].m_label     = NODISPLAY;
  g.m_sides[SOUTH].m_label    = itos(iBase-2) + MARKER + LEASTSIG;;
  g.m_sides[WEST].m_label     = std::string(COPY) + MARKER;
  ts.push_back(g);

  //  Tiles in LSB position with marker value != (iBase-1) and just incrementing
  for (int i=1; i<iBase-1; i++) {
    Tile g;
    g.m_display                 = itos(i) + MARKER + LEASTSIG;
    g.m_sides[NORTH].m_strength = 2;
    g.m_sides[EAST].m_strength  = 0;
    g.m_sides[SOUTH].m_strength = 2;
    g.m_sides[WEST].m_strength  = 1;
    g.m_sides[NORTH].m_label    = itos(i) + MARKER + LEASTSIG;
    g.m_sides[EAST].m_label     = NODISPLAY;
    g.m_sides[SOUTH].m_label    = itos(i-1) + MARKER + LEASTSIG;;
    g.m_sides[WEST].m_label     = std::string(COPY);
    ts.push_back(g);
  }

  //  Tiles in MSB position that receive the MARKER from the right
  for (int i=1; i<iBase; i++) {
    Tile a;
    a.m_display                 = std::string(MOSTSIG) + itos(i) + MARKER;
    a.m_sides[NORTH].m_strength = 2;
    a.m_sides[EAST].m_strength  = 1;
    a.m_sides[SOUTH].m_strength = 1;
    a.m_sides[WEST].m_strength  = 0;
    a.m_sides[NORTH].m_label    = std::string(MOSTSIG) + itos(i) + MARKER;
    a.m_sides[EAST].m_label     = std::string(COPY) + MARKER;
    a.m_sides[SOUTH].m_label    = std::string(MOSTSIG) + itos(i);
    a.m_sides[WEST].m_label     = NODISPLAY;
    ts.push_back(a);
  }

  //  Tiles in MSB position that increment and pass the MARKER back to the right
  for (int i=1; i<iBase; i++) {
    Tile a;
    a.m_display                 = std::string(MOSTSIG) + itos(i);
    a.m_sides[NORTH].m_strength = 1;
    a.m_sides[EAST].m_strength  = 1;
    a.m_sides[SOUTH].m_strength = 2;
    a.m_sides[WEST].m_strength  = 0;
    a.m_sides[NORTH].m_label    = std::string(MOSTSIG) + itos(i);
    a.m_sides[EAST].m_label     = MARKER;
    a.m_sides[SOUTH].m_label    = std::string(MOSTSIG) + itos(i-1) + MARKER;
    a.m_sides[WEST].m_label     = NODISPLAY;
    ts.push_back(a);
  }

  if (iType == COUNTER_LOGWIDTH) {
    //  Tile that initiates growth of a new MSB column (but isn't in LSB position)
    Tile d;
    d.m_display                 = itos(0);
    d.m_sides[NORTH].m_strength = 1;
    d.m_sides[EAST].m_strength  = 1;
    d.m_sides[SOUTH].m_strength = 2;
    d.m_sides[WEST].m_strength  = 2;
    d.m_sides[NORTH].m_label    = itos(0);
    d.m_sides[EAST].m_label     = std::string(MARKER);
    d.m_sides[SOUTH].m_label    = std::string(MOSTSIG) + itos(iBase-1) + MARKER;
    d.m_sides[WEST].m_label     = std::string(MOSTSIG);
    ts.push_back(d);
  } else {
    //  MSB tile in fixed-width counter with value 0 that gets MARKER from the right
    Tile d;
    d.m_display                 = std::string(MOSTSIG) + itos(0) + MARKER;
    d.m_sides[NORTH].m_strength = 2;
    d.m_sides[EAST].m_strength  = 1;
    d.m_sides[SOUTH].m_strength = 1;
    d.m_sides[WEST].m_strength  = 0;
    d.m_sides[NORTH].m_label    = std::string(MOSTSIG) + itos(0) + MARKER;
    d.m_sides[EAST].m_label     = std::string(COPY) + std::string(MARKER);
    d.m_sides[SOUTH].m_label    = std::string(MOSTSIG) + itos(0);
    d.m_sides[WEST].m_label     = NODISPLAY;
    ts.push_back(d);

    //  MSB tile in fixed-width counter that flips back to 0
    Tile e;
    e.m_display                 = std::string(MOSTSIG) + itos(0);
    e.m_sides[NORTH].m_strength = 1;
    e.m_sides[EAST].m_strength  = 1;
    e.m_sides[SOUTH].m_strength = 2;
    e.m_sides[WEST].m_strength  = 0;
    e.m_sides[NORTH].m_label    = std::string(MOSTSIG) + itos(0);
    e.m_sides[EAST].m_label     = std::string(MARKER);
    e.m_sides[SOUTH].m_label    = std::string(MOSTSIG) + itos(iBase-1) + MARKER;
    e.m_sides[WEST].m_label     = NODISPLAY;
    ts.push_back(e);
  }

  //  Tiles in LSB position that receive the MARKER from the left
  for (int i=0; i<iBase; i++) {
    Tile a;
    a.m_display                 = itos(0) + MARKER + LEASTSIG;
    a.m_sides[NORTH].m_strength = 2;
    a.m_sides[EAST].m_strength  = 0;
    a.m_sides[SOUTH].m_strength = 1;
    a.m_sides[WEST].m_strength  = 1;
    a.m_sides[NORTH].m_label    = itos(0) + MARKER + LEASTSIG;
    a.m_sides[EAST].m_label     = NODISPLAY;
    a.m_sides[SOUTH].m_label    = itos(i) + LEASTSIG;
    a.m_sides[WEST].m_label     = MARKER;
    ts.push_back(a);
  }

  //  Tiles in interior positions that receive the MARKER from the right
  for (int i=0; i<iBase-1; i++) {
    Tile a;
    a.m_display                 = itos(i) + MARKER;
    a.m_sides[NORTH].m_strength = 2;
    a.m_sides[EAST].m_strength  = 1;
    a.m_sides[SOUTH].m_strength = 1;
    a.m_sides[WEST].m_strength  = 1;
    a.m_sides[NORTH].m_label    = itos(i) + MARKER;
    a.m_sides[EAST].m_label     = std::string(COPY) + MARKER;
    a.m_sides[SOUTH].m_label    = itos(i);
    a.m_sides[WEST].m_label     = COPY;
    ts.push_back(a);
  }

  //  Tiles in interior positions & MSB position that receive COPY from the right
  for (int i=0; i<iBase; i++) {
    Tile a;
    a.m_display                 = itos(i);
    a.m_sides[NORTH].m_strength = 1;
    a.m_sides[EAST].m_strength  = 1;
    a.m_sides[SOUTH].m_strength = 1;
    a.m_sides[WEST].m_strength  = 1;
    a.m_sides[NORTH].m_label    = itos(i);
    a.m_sides[EAST].m_label     = COPY;
    a.m_sides[SOUTH].m_label    = itos(i);
    a.m_sides[WEST].m_label     = COPY;
    ts.push_back(a);

    Tile b;
    b.m_display                 = MOSTSIG + itos(i);
    b.m_sides[NORTH].m_strength = 1;
    b.m_sides[EAST].m_strength  = 1;
    b.m_sides[SOUTH].m_strength = 1;
    b.m_sides[WEST].m_strength  = 0;
    b.m_sides[NORTH].m_label    = MOSTSIG + itos(i);
    b.m_sides[EAST].m_label     = COPY;
    b.m_sides[SOUTH].m_label    = MOSTSIG + itos(i);
    b.m_sides[WEST].m_label     = NODISPLAY;
    ts.push_back(b);
  }

  //  Tiles in interior positions that are zeroed and pass the MARKER back to the right
  for (int i=0; i<iBase; i++) {
    Tile a;
    a.m_display                 = itos(0);
    a.m_sides[NORTH].m_strength = 1;
    a.m_sides[EAST].m_strength  = 1;
    a.m_sides[SOUTH].m_strength = 1;
    a.m_sides[WEST].m_strength  = 1;
    a.m_sides[NORTH].m_label    = itos(0);
    a.m_sides[EAST].m_label     = MARKER;
    a.m_sides[SOUTH].m_label    = itos(i);
    a.m_sides[WEST].m_label     = MARKER;
    ts.push_back(a);
  }

  //  Tiles in interior positions that increment and pass the MARKER back to the right
  for (int i=0; i<iBase-1; i++) {
    Tile a;
    a.m_display                 = itos(i+1);
    a.m_sides[NORTH].m_strength = 1;
    a.m_sides[EAST].m_strength  = 1;
    a.m_sides[SOUTH].m_strength = 2;
    a.m_sides[WEST].m_strength  = 1;
    a.m_sides[NORTH].m_label    = itos(i+1);
    a.m_sides[EAST].m_label     = MARKER;
    a.m_sides[SOUTH].m_label    = itos(i) + MARKER;
    a.m_sides[WEST].m_label     = COPY;
    ts.push_back(a);
  }

  //  Tiles in interior positions and equal to (base-1) and which just continue passing the MARKER to the left
  Tile h;
  h.m_display                 = itos(iBase-1);
  h.m_sides[NORTH].m_strength = 1;
  h.m_sides[EAST].m_strength  = 1;
  h.m_sides[SOUTH].m_strength = 1;
  h.m_sides[WEST].m_strength  = 1;
  h.m_sides[NORTH].m_label    = itos(iBase-1);
  h.m_sides[EAST].m_label     = std::string(COPY) + MARKER;
  h.m_sides[SOUTH].m_label    = itos(iBase-1);
  h.m_sides[WEST].m_label     = std::string(COPY) + MARKER;
  ts.push_back(h);


  //////////////////////////////////////////////////////////////////////
  //  Rotate the tiles as necessary
  for (size_t i=0; i<ts.size(); i++) {
    ts[i].Rotate(iDirGrow*90);
  }

  //////////////////////////////////////////////////////////////////////
  //  Reflect the tiles as necessary
  if (iDirFace != ((iDirGrow + 1) % 4)) {
    for (size_t i=0; i<ts.size(); i++) {
      if ((iDirGrow == NORTH) || (iDirGrow == SOUTH)) {
        ts[i].Reflect(VERTICAL);
      } else {
        ts[i].Reflect(HORIZONTAL);
      }
    }
  }

  *pTS += ts;
  return true;
}
