/////////////////////////////////////////////////////////////////////////////
// Name:        fiber.h
// Purpose:     Generate TAM tilesets which self-assemble into fibered fractals
// Author:      Matthew Patitz <mpatitz@yahoo.com>
// Modified by:
// Created:     Nov/2007
// Copyright:   (c) 2008, Matthew John Patitz
// Licence:     GPL
/////////////////////////////////////////////////////////////////////////////

#ifndef __FIBER_H__
#define __FIBER_H__

#include "tiles.h"
#include "counters.h"

#define MOSTSIGNOGROW     "m"
#define GROWBACK          "B"
#define SPACERMARKER      "#"
#define TRANSITIONMARKER  "@"
#define SPACERCOPYLEFT    "<"
#define SPACERCOPYRIGHT   ">"
#define TOBEDETERMINED    "?"
#define GROWFIBERMARKER   "!"

#define QUAD1   0
#define QUAD2   1
#define QUAD3   2
#define QUAD4   3

#define SPACERCOLOR            "RGB(127,0,0)"
#define TRANSITIONSPACERCOLOR  "RGB(100,100,100)"


int GetReversedPosition(int pos, int base);
std::string GenerateTileName(Point pt, int iSetNum, bool bWithPrefix = false);
std::string GetPrefixFromNum(int num, int iDirGrow, PointSet* pPS);
int TranslateDirection(int iDir, int iDirGrow, int iDirFace);
int ConvertLocationToType(int i, int iDirGrow, PointSet* pPS);
std::string BraceNumber(int i, int iDirGrow, PointSet* pPS);

TileSet CreateCounterTileSet(int iBase, int iType, int iOrientation, PointSet* pPS);
TileSet CreateTransitionSpacerTileSet(int iBase, int iType, PointSet* pPS);
TileSet CreateReverseTransitionTileSet(int iBase, int iType, PointSet* pPS);

bool GenerateFiberedSSFTileset(PointSet* pPS, TileSet* pTS, int quadrant);


#endif  // __FIBER_H__