/////////////////////////////////////////////////////////////////////////////
// Name:        points.cpp
// Purpose:     Helper classes for points and sets of points
// Author:      Matthew Patitz <mpatitz@yahoo.com>
// Modified by:
// Created:     Nov/2007
// Copyright:   (c) 2008, Matthew John Patitz
// Licence:     GPL
/////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "points.h"
#include "tiles.h"


bool PointSet::GetPointsFromFile(std::string strFilePath)
{
  FILE* fp = fopen(strFilePath.c_str(), "r");
  if (!fp) {
    return false;
  }

  std::string sBuff;
  while (!feof(fp)) {
    char buff[4096] = {0};
    fread(buff, sizeof(char), 4095, fp) ;
    sBuff += buff;

  }
  fclose(fp);

  char* f = (char*)sBuff.c_str();
  int num = 1;
  while (*f) {
    while ((*f != '(') && *f) f++;    //  Find an opening paren
    if (*f == '(') {
      f++;
      int x = atoi(f);
      while (*f != ',') f++;
      f++;
      int y = atoi(f);
      while (*f != ')') f++;
      f++;
      Point pt(x, y, num++);
      for (size_t i=0; i<size(); i++) {
        if (pt == at(i)) {
          std::cout << "\nERROR:  Duplicate point (" + itos(x) + "," + itos(y) + ") specified.  Aborting.\n\n";
          return false;
        }
      }
      push_back(pt);
    }    
  }

  return true;
}

int PointSet::GetPointNumForLocation(Point& pt)
{
  for (size_t i=0; i<size(); i++) {
    if (at(i) == pt) {
      return (int)i;
    }
  }

  return -1;
}

bool PointSet::CalculateInputs(bool bDebug)
{
  if (m_origin == -1) {
    if (bDebug) {
      std::cout << "ERROR:  Can't calculate inputs for points because no origin has been set.\n";
    }
    return false;
  }

  PointSet inputPts;
  inputPts.push_back(at(m_origin));
  size_t donePoints = 1;

  while (donePoints != size()) {
    size_t tmpDone = donePoints;
    for (size_t i=0; i<size(); i++) {
      int ptNum = inputPts.GetPointNumForLocation(at(i));
      if (ptNum == -1) {
        ptNum = inputPts.GetPointNumForLocation(at(i).GetOffset(SOUTH, 1));
        if (ptNum >= 0) {
          at(i).SetInput(&at(GetPointNumForLocation(at(i).GetOffset(SOUTH, 1))));
          at(i).SetInputDir(SOUTH);
          if (bDebug) {
            std::cout << "(" + itos(at(i).GetX()) + "," + itos(at(i).GetY()) + ") grows from " + DirectionStringFromInt(SOUTH).c_str() + "\n";
          }
          inputPts.push_back(at(i));
          donePoints++;
          break;
        }

        ptNum = inputPts.GetPointNumForLocation(at(i).GetOffset(WEST, 1));
        if (ptNum >= 0) {
          at(i).SetInput(&at(GetPointNumForLocation(at(i).GetOffset(WEST, 1))));
          at(i).SetInputDir(WEST);
          if (bDebug) {
            std::cout << "(" + itos(at(i).GetX()) + "," + itos(at(i).GetY()) + ") grows from " + DirectionStringFromInt(WEST).c_str() + "\n";
          }
          inputPts.push_back(at(i));
          donePoints++;
          break;
        }

        ptNum = inputPts.GetPointNumForLocation(at(i).GetOffset(NORTH, 1));
        if (ptNum >= 0) {
          at(i).SetInput(&at(GetPointNumForLocation(at(i).GetOffset(NORTH, 1))));
          at(i).SetInputDir(NORTH);
          if (bDebug) {
            std::cout << "(" + itos(at(i).GetX()) + "," + itos(at(i).GetY()) + ") grows from " + DirectionStringFromInt(NORTH).c_str() + "\n";
          }
          inputPts.push_back(at(i));
          donePoints++;
          break;
        }

        ptNum = inputPts.GetPointNumForLocation(at(i).GetOffset(EAST, 1));
        if (ptNum >= 0) {
          at(i).SetInput(&at(GetPointNumForLocation(at(i).GetOffset(EAST, 1))));
          at(i).SetInputDir(EAST);
          if (bDebug) {
            std::cout << "(" + itos(at(i).GetX()) + "," + itos(at(i).GetY()) + ") grows from " + DirectionStringFromInt(EAST).c_str() + "\n";
          }
          inputPts.push_back(at(i));
          donePoints++;
          break;
        }
      }
    }
  }

/*
  while (donePoints != size()) {
    size_t tmpDone = donePoints;

    for (size_t i=0; i<inputPts.size(); i++) {
      Point ptN = inputPts.at(i).GetOffset(NORTH, 1);
      int num = GetPointNumForLocation(ptN);
      if ((num >= 0) && (inputPts.GetPointNumForLocation(ptN) < 0)) {
        at(num).SetInput(&at(GetPointNumForLocation(inputPts.at(i))));
        at(num).SetInputDir(SOUTH);
        if (bDebug) {
          std::cout << "(" + itos(at(num).GetX()) + "," + itos(at(num).GetY()) + ") grows from " + DirectionStringFromInt(SOUTH).c_str() + "\n";
        }
        inputPts.push_back(at(num));
        donePoints++;
      }

      Point ptE = inputPts.at(i).GetOffset(EAST, 1);
      num = GetPointNumForLocation(ptE);
      if ((num >= 0) && (inputPts.GetPointNumForLocation(ptE) < 0)) {
        at(num).SetInput(&at(GetPointNumForLocation(inputPts.at(i))));
        at(num).SetInputDir(WEST);
        if (bDebug) {
          std::cout << "(" + itos(at(num).GetX()) + "," + itos(at(num).GetY()) + ") grows from " + DirectionStringFromInt(WEST).c_str() + "\n";
        }
        inputPts.push_back(at(num));
        donePoints++;
      }

      Point ptS = inputPts.at(i).GetOffset(SOUTH, 1);
      num = GetPointNumForLocation(ptS);
      if ((num >= 0) && (inputPts.GetPointNumForLocation(ptS) < 0)) {
        at(num).SetInput(&at(GetPointNumForLocation(inputPts.at(i))));
        at(num).SetInputDir(NORTH);
        if (bDebug) {
          std::cout << "(" + itos(at(num).GetX()) + "," + itos(at(num).GetY()) + ") grows from " + DirectionStringFromInt(NORTH).c_str() + "\n";
        }
        inputPts.push_back(at(num));
        donePoints++;
      }

      Point ptW = inputPts.at(i).GetOffset(WEST, 1);
      num = GetPointNumForLocation(ptW);
      if ((num >= 0) && (inputPts.GetPointNumForLocation(ptW) < 0)) {
        at(num).SetInput(&at(GetPointNumForLocation(inputPts.at(i))));
        at(num).SetInputDir(EAST);
        if (bDebug) {
          std::cout << "(" + itos(at(num).GetX()) + "," + itos(at(num).GetY()) + ") grows from " + DirectionStringFromInt(EAST).c_str() + "\n";
        }
        inputPts.push_back(at(num));
        donePoints++;
      }
    }

    if (tmpDone == donePoints) {
      int dn = size() - inputPts.size();
      std::cout << itos(dn) + " points are disconnected from the origin:\n";
      for (size_t i=0; i<size(); i++) {
        int ptNum = inputPts.GetPointNumForLocation(at(i));
        if (ptNum == -1) {
          std::cout << "(" + itos(at(i).GetX()) + "," + itos(at(i).GetY()) + ") ";
        }
      }
      std::cout << "\n";
      return false;
    }
  }
*/
  return true;
}

void PointSet::CalculateOutputs()
{
  for (size_t i=0; i<size(); i++) {
    for (size_t j=0; j<size(); j++) {
      Point* pPt = at(j).GetInput();
      if (pPt) {
        int in2 = GetPointNumForLocation(*pPt);
        if (in2 == i) {
          int dir = at(i).GetDirectionToPoint(at(j));
          at(i).SetOutput(dir, &at(j));
        }
      }
    }
  }
}

bool PointSet::PrintToFile(const char* fileName)
{
  std::ofstream out(fileName);
  if (!out.is_open()) {
    return false;
  }

  for (size_t j=0; j<size(); j++) {
    out << "(";
    out << at(j).ToString();
    out << ") ";
  }
  out.close();

  return true;
}

const PointSet& PointSet::operator+=(const PointSet& ps)
{
  for (size_t i=0; i<ps.size(); i++) {
    push_back(ps[i]);
  }
  return *this;
}

const PointSet PointSet::operator+(const Point& pt) const
{
  PointSet ps;
  for (size_t i=0; i<size(); i++) {
    ps.push_back(at(i) + pt);
  }
  return ps;
}

int PointSet::RemoveDuplicates()
{
  int dups = 0;

  PointSet newPS;
  for (size_t i=0; i<size(); i++) {
    bool bFound = false;
    for (size_t j=i+1; j<size(); j++) {
      if (at(i) == at(j)) {
        bFound = true;
        break;
      }
    }
    if (!bFound) {
      newPS.push_back(at(i));
    } else {
      dups++;
    }
  }

  clear();
  *this = newPS;

  return dups;
}

void Point::ConstructorHelper(int x, int y, int num)
{
  m_x   = x;
  m_y   = y;
  m_num = num;

  m_pPtInput = NULL;
  for (int i=NORTH; i<=WEST; i++) {
    m_pPtOutputs[i] = NULL;
  }
}

Point Point::GetOffset(int dir, int magnitude)
{
  Point ptRet(m_x, m_y);
  if (dir == NORTH) {
    ptRet.SetY(ptRet.GetY() + magnitude);
  } else if (dir == EAST) {
    ptRet.SetX(ptRet.GetX() + magnitude);
  } else if (dir == SOUTH) {
    ptRet.SetY(ptRet.GetY() - magnitude);
  } else if (dir == WEST) {
    ptRet.SetX(ptRet.GetX() - magnitude);
  }
  return ptRet;
}

int Point::GetDirectionToPoint(Point ptIn)
{
  if (ptIn.GetX() > m_x) {
    return EAST;
  } else if (ptIn.GetY() > m_y) {
    return NORTH;
  } else if (ptIn.GetX() < m_x) {
    return WEST;
  } else if (ptIn.GetY() < m_y) {
    return SOUTH;
  } else {
    return -1;
  }
}

int Point::GetDistanceToPoint(Point ptIn)
{
  int dist = ((m_x - ptIn.GetX()) > 0) ? (m_x - ptIn.GetX()) : (ptIn.GetX() - m_x);
  dist += ((m_y - ptIn.GetY()) > 0) ? (m_y - ptIn.GetY()) : (ptIn.GetY() - m_y);
  return dist;
}

std::string Point::ToString()
{
  std::string sRet = itos(m_x) + std::string(",") + itos(m_y);
  return sRet;
}

