/////////////////////////////////////////////////////////////////////////////
// Name:        points.h
// Purpose:     Helper classes for points and sets of points
// Author:      Matthew Patitz <mpatitz@yahoo.com>
// Modified by:
// Created:     Nov/2007
// Copyright:   (c) 2008, Matthew John Patitz
// Licence:     GPL
/////////////////////////////////////////////////////////////////////////////

#ifndef __POINTS_H__
#define __POINTS_H__

#include <vector>


class Point
{
public:
  Point() { ConstructorHelper(-1, -1, -1); };
  Point(int x, int y) { ConstructorHelper(x, y, -1); };
  Point(int x, int y, int num) { ConstructorHelper(x, y, num); };
  bool IsValid() { return (m_x != -1 && m_y != -1); };
  bool operator==(const Point& ptIn) { return (m_x == ptIn.m_x && m_y == ptIn.m_y); };
  bool operator!=(const Point& ptIn) { return !(*this == ptIn); };
  const Point& operator=(const Point& ptIn) { m_x = ptIn.m_x; m_y = ptIn.m_y; return *this; };
  const Point& operator+=(const Point& ptIn) { m_x += ptIn.m_x; m_y += ptIn.m_y; return *this; };
  const Point operator+(const Point& ptIn) const { return Point(m_x + ptIn.m_x, m_y + ptIn.m_y); };
  const Point& operator*=(const Point& ptIn) { m_x = m_x * ptIn.m_x; m_y = m_y * ptIn.m_y; return *this; };
  const Point& operator*=(int fact) { m_x = m_x * fact; m_y = m_y * fact; return *this; };

  Point GetOffset(int dir, int magnitude);
  int GetDirectionToPoint(Point ptIn);    //  This will only work for points that have either the same x or y coord
  int GetDistanceToPoint(Point ptIn);
  std::string ToString();

  int GetX() { return m_x; };
  int GetY() { return m_y; };
  void SetX(int x) { m_x = x; };
  void SetY(int y) { m_y = y; };
  Point* GetInput() { return m_pPtInput; };
  void SetInput(Point* ptIn) { m_pPtInput = ptIn; };
  int GetInputDir() { return m_dirInput; };
  void SetInputDir(int dir) { m_dirInput = dir; };
  Point* GetOutput(int dir) { return m_pPtOutputs[dir]; };
  void SetOutput(int dir, Point* ptOut) { m_pPtOutputs[dir] = ptOut; };
  int GetNum() { return m_num; };
  void SetNum(int num) { m_num = num; };

protected:
  void ConstructorHelper(int x, int y, int num);

  int     m_x;
  int     m_y;
  int     m_num;
  int     m_dirInput;
  Point*  m_pPtInput;
  Point*  m_pPtOutputs[4];
};

class PointSet : public std::vector<Point>
{
public:
  PointSet() { m_origin = -1; };

  const PointSet& operator+=(const PointSet& ps);
  const PointSet operator+(const Point& pt) const;

  void Clear() { clear(); m_origin = -1; };
  int RemoveDuplicates();

  int GetPointNumForLocation(Point& pt);
  bool CalculateInputs(bool bDebug = false);
  void CalculateOutputs();

  bool GetPointsFromFile(std::string strFilePath);
  
  void SetOrigin(int origin) { m_origin = origin; };
  int GetOrigin() { return m_origin; };

  bool PrintToFile(const char* fileName);

protected:
  int m_origin;
};


#endif  // __POINTS_H__