/////////////////////////////////////////////////////////////////////////////
// Name:        tiles.h
// Purpose:     Classes and helper functions to work with tiletype definitions
// Author:      Matthew Patitz <mpatitz@yahoo.com>
// Modified by:
// Created:     Nov/2007
// Copyright:   (c) 2008, Matthew John Patitz
// Licence:     GPL
/////////////////////////////////////////////////////////////////////////////

#ifndef __TILE_H__
#define __TILE_H__

#include <string>
#include <vector>
#include <fstream>
#include <iostream>
#include <cstdarg>

#define NORTH             0
#define EAST              1
#define SOUTH             2
#define WEST              3

#define HORIZONTAL        0
#define VERTICAL          1

#define NODISPLAY         "-99"
#define NAMEDELIMIT       "|"

#define TREND_TILETYPES    0
#define TILESIM_TILETYPES  1


class StringSet : public std::vector<std::string>
{
public:
  StringSet(std::string& str) { push_back(str); };
  StringSet(const char* s) { push_back(s); };
  StringSet(const char* fmt, ...);
  std::vector<std::string> StringSetHelper(std::vector<std::vector<std::string>>& vvs, size_t i);
};

//  Class to represent one side of a tile
class TileSide
{
public:
  TileSide() { m_strength = 0; m_label = NODISPLAY; m_bAddPrefix = true; };
  TileSide(const TileSide& ts) { m_label = ts.m_label; m_strength = ts.m_strength; m_bAddPrefix = ts.m_bAddPrefix; };
  std::string   m_label;
  int           m_strength;
  bool          m_bAddPrefix;
};
class TileSides : public std::vector<TileSide>
{
public:
  TileSides();
};

//  Class to represent entire tile type
class Tile
{
public:
  Tile() {};
  Tile(const Tile& t);
  Tile(const char* name) { m_name = name; };

  void Print(std::ofstream* o);
  const char* GetName() { return m_name.c_str(); };
  void SetName(const char* name) { m_name = name; };
  void GenerateName();
  void Rotate(int degrees);       //  Rotation is done in the clockwise direction
  void Reflect(int axis);         //  Refect the tile across the horizontal or vertical axis
  void UsePrefix(bool bUse);

  std::string   m_display;
  std::string   m_prefix;
  std::string   m_suffix;
  TileSides     m_sides;
  std::string   m_color;
protected:
  std::string   m_name;
};

class TileSet : public std::vector<Tile>
{
public:
  TileSet() {};
  TileSet(StringSet* ssDisplay, int iNorthStrength, StringSet* ssNorthGlues, int iEastStrength, StringSet* ssEastGlues,
                                int iSouthStrength, StringSet* ssSouthGlues, int iWestStrength, StringSet* ssWestGlues);
  TileSet& operator+(const TileSet& ts);
  TileSet& operator+=(const TileSet& ts);

  bool PrintToFile(const char* fileName, int type = TREND_TILETYPES);

  Tile* GetTile(std::string sName);

  void SetPrefix(std::string sPrefix);
  void SetSuffix(std::string sSuffix);

  void SetColor(std::string color);
};


//  Helper functions

std::string itos(int i);
void SplitString(std::string str, std::string delim, std::vector<std::string> &results);
std::string StripSpaces(std::string& str);

int GetOrientationFromDirection(int dir);
int OppositeOrientation(int orient);

int DirectionIntFromString(std::string sDir);
std::string DirectionStringFromInt(int iDir);
int OppositeDirection(int dir);

#endif  // __TILE_H__